//   This software is available only for non-commercial  amateur radio
//   or educational  applications.   ALL  other  uses  are  prohibited.
//   This  software may  be  modified  only  if the  resulting code  be
//   made available  publicly  and the  original  author given  credit.
//   Contact the author for information on commercial licenses.
//   (c) Copyright 2000,  Robert Jeffrey King, WB8WKA.  All rights reserved.

///////////////////////////////////////////////////////////////////////////
//// Library for a Vector 2X fluxgate compass configured for slave     ////
////                                                                   ////
////   reset_vector2x();    Reset Vector compass to default            ////
////                                                                   ////
////   long read_vector2x(void); read long containing heading       ////
////                                                                   ////
////////////////////////////////////////////////////////////////////////////
/*Compass Bearing.
See Figure 1A.
The general theory of the device is that two perpendicularly mounted sensors detect the magnitude of the magnetic field. Of course, if the unit is mounted upside down, the field is reversed in each sensor. Thus, Precision Navigation has provided two ter minals /XFLIP and YFLIP. For normal operation, /XFLIP is set to logic one and YFLIP to logic zero.
The module may be operated in either a Master or Slave mode. As our intent was to interface this with a processor, we operated in the slave mode, where the processor controls the clocking of the data. Thus input M/S was set to logic zero.
Within the slave mode, there is an option for either the raw data measured by each of the two sensors to be returned in two 16-bit words or for the compass bearing to be returned. As we are considering taking a compass bearing at the moment, the /RAW inp ut is set to logic one. The RAW data mode is treated elsewhere.
The /LO_RES input permits high (logic one) and low resolution. In measuring a compass bearing , we found the low resolution more than adequate.
The /CAL input associated with calibration which is discussed elsewhere. As we are now concerned with making a measurement, this input is high. The CI output is a calibration indicator and is left unconnected in this application.
There is an option for the compass to return the compass bearing either in BCD or natural binary format. In both cases, the bearing consists of 16 bits with the most significant six bits set 0. Thus, in the BCD mode the lower ten bits are a binary code d decimal representation of the compass bearing. In the natural binary mode, only the lower nine bits are used. Consider a compass bearing of 359 degrees;

          BCD      0000 0011 0101 1001 or 0x0359
          binary   0000 0001 0 110 0101 or 0x0165
In our application, we used the BCD mode and thus the /BCD input was set to logic zero.
Module inputs; /RESET, SCLK, /SS and /PC and outputs SDO and EOC are used to obtain a reading. Please refer to the timing diagram on Figure #1B.
Note that /RESET is normally at logic one. It is brought to a momentary low for a minimum of 10 ms to reset the device.
The /P/C (polled/continuous) input is brought momentarily low for a minimum of 10 ms to initiate the sequence. The module brings EOC (end of conversion) low during the time it is making the conversion and goes high to signal the external processor that t he conversion is complete. Input /SS (slave select) is then brought low and a minimum of 10 msec thereafter, sixteen negative going clock pulses are output to the module, with the data on SDO (serial data out) being read after each of the positive SCLK t ransitions.
This is implemented in the following routine COMPASS1.C.

*/

void reset(void);
void set_ss(int state);
void wink_pc_low(void);
void clock(void);
long get_reading_bcd(void);

void reset_both(void);
void set_ss_2(int state);
void wink_pc_low_both(void);
void clock_2(void);
long get_reading_bcd_2(void);

void reset(void)
/* brings reset momentarily low - all others high */
{
	output_high(VEC1_RESET);			// Pull low to reset
	output_high(VEC1_SELECT);			// This must be high to reset
	output_high(VEC1_CLOCK);
	output_high(VEC1_PC);
	delay_ms(1);
	output_low(VEC1_RESET);				// Pull low to reset
	delay_ms(50);						// Hold in reset for 20ms
	output_high(VEC1_RESET);
	delay_ms(500);
}

void reset_both(void)
/* brings reset momentarily low - all others high */
{
	output_high(VEC1_RESET);			// Pull low to reset
	output_high(VEC1_SELECT);			// This must be high to reset
	output_high(VEC1_CLOCK);
	output_high(VEC1_PC);
	output_high(VEC2_RESET);			// Pull low to reset
	output_high(VEC2_SELECT);			// This must be high to reset
	output_high(VEC2_CLOCK);
	output_high(VEC2_PC);
	delay_ms(1);
	output_low(VEC1_RESET);				// Pull low to reset
	output_low(VEC2_RESET);				// Pull low to reset
	delay_ms(50);						// Hold in reset for 20ms
	output_high(VEC1_RESET);
	output_high(VEC2_RESET);
	delay_ms(500);
}

void wink_pc_low(void)
{
	output_low(VEC1_PC);				// Bring P/C/ low to poll vector for one readering
	delay_ms(20);						// Wait at least 10ms after bringing low
	output_high(VEC1_PC);				// Bring high
}

void wink_pc_low_both(void)
{
	output_low(VEC1_PC);				// Bring P/C/ low to poll vector for one readering
	output_low(VEC2_PC);				// Bring P/C/ low to poll vector for one readering
	delay_ms(20);						// Wait at least 10ms after bringing low
	output_high(VEC1_PC);				// Bring high
	output_high(VEC2_PC);				// Bring high
}

void set_ss(int state)
/* set /ss to indicated state */
{
	if(state == 1)
	{
		output_high(VEC1_SELECT);
	}
	else
	{
		output_low(VEC1_SELECT);
	}
}
void set_ss_2(int state)
/* set /ss to indicated state */
{
	if(state == 1)
	{
		output_high(VEC2_SELECT);
	}
	else
	{
		output_low(VEC2_SELECT);
	}
}

long get_reading_bcd(void)
{
	int clock_pulses = 16;
	long reading=0, bit;

//	for(clock_pulses=15; clock_pulses < 1; clock_pulses--)
//	for(clock_pulses=0; clock_pulses < 16; clock_pulses++)
	While (clock_pulses)
	{
		clock();
		clock_pulses -= 1;
		bit = input(VEC1_DATA);
		reading = reading | (bit << clock_pulses);
	}
	return(reading);
}

long get_reading_bcd_2(void)
{
	int clock_pulses = 16;
	long reading=0, bit;

//	for(clock_pulses=15; clock_pulses < 1; clock_pulses--)
//	for(clock_pulses=0; clock_pulses < 16; clock_pulses++)
	While (clock_pulses)
	{
		clock_2();
		clock_pulses -= 1;
		bit = input(VEC2_DATA);
		reading = reading | (bit << clock_pulses);
	}
	return(reading);
}

void clock(void)
{
	output_low(VEC1_CLOCK);		// SCLK low
	delay_us(20);
	output_high(VEC1_CLOCK);	// SCLK high
	delay_us(20);
}

void clock_2(void)
{
	output_low(VEC2_CLOCK);		// SCLK low
	delay_us(20);
	output_high(VEC2_CLOCK);	// SCLK high
	delay_us(20);
}